#define SUCCESS 0
#define MODULE_ERROR -1
#define TIMEOUT_ERROR -2

const unsigned char BT_STX = '+'; // New command start
const unsigned char BT_ETX = ';'; // Command end
const unsigned char BT_CNK = 'K'; // Connect request/confermation
const unsigned char BT_DNK = 'D'; // Disconnect request/confermation
const unsigned char BT_QRY = 'Q'; // Query status from previously-set
const unsigned char BT_PST = 'P'; // Preset effect
const unsigned char BT_SST = 'S'; // Preset effect
const unsigned char BT_FST = 'F'; // Preset effect
const unsigned char BT_MDL = ':'; // Preset divider, new element within the sequence is following
const unsigned char BT_RGB = 'C'; // Static color
const unsigned char BT_RGW = 'B'; // Static color for white
const unsigned char BT_WHT = 'W'; // White intensity
const unsigned char BT_EFT = 'E'; // Personalized effect
const unsigned char BT_ERR = 'R'; // Resend last command
String EOL = String("\n\r");//String (0x0A)+ String (0x0D);


short reset_BC127()
{
  String buffer;
  
  knownStart();
  
  // Now issue the reset command.
  Serial.print("RESET\r");
  Serial.flush();
  
  // We're going to use the internal timer to track the elapsed time since we
  //  issued the reset. Bog-standard Arduino stuff.
  unsigned long resetStart = millis();
  
  // This is our timeout loop. We'll give the module 2 seconds to reset.
  while ((resetStart + 2000) > millis())
  {
    // Grow the current buffered data, until we receive the EOL string.    
    if (Serial.available() > 0) 
    {
      char temp = char(Serial.read());
      buffer.concat(temp);
    }
    
    if (buffer.endsWith(EOL))
    {
      if (buffer.startsWith(String("ER"))){return MODULE_ERROR;}
      if (buffer.startsWith(String("Re"))){return SUCCESS;}
      buffer = "";
    }    
  }
  return TIMEOUT_ERROR;
}

short knownStart()
{
  String buffer;
  
  Serial.print("\r");
  Serial.flush();
  
  // We're going to use the internal timer to track the elapsed time since we
  //  issued the reset. Excape-standard Arduino stuff.
  unsigned long startTime = millis();
  
  // This is our timeout loop. We're going to give our module 100ms to come up
  //  with a new character, and return with a timeout failure otherwise.
  while (buffer.endsWith(EOL) != true)
  {
    // Purge the serial data received from the module, along with any data in
    //  the buffer at the time this command was sent.
    if (Serial.available() >0) 
    {
      buffer.concat(char(Serial.read()));
      startTime = millis();
    }
    if ((startTime + 1000) < millis()){return TIMEOUT_ERROR;}
  }
  if (buffer.startsWith(String("ERR")))
  {
    return SUCCESS;
  }
  else{return SUCCESS; }
}

short send_cromatica_CMD(char CMD)
{
  String buffer;
 
  knownStart(); // Clear the serial buffer in the module and the Arduino.

//  Serial.print("SEND 12 +K;\r");
//  Serial.flush();
  Serial.print("SEND 12 +");  
  Serial.print(CMD);
  Serial.print(";\r");
  Serial.flush();
  
  // We're going to use the internal timer to track the elapsed time since we
  //  issued the command. Excape-standard Arduino stuff.
  unsigned long startTime = millis();
    
  // This is our timeout loop. We'll give the module 1 second.
  while ((startTime + 1000) > millis())
  {
    // Grow the current buffered data, until we receive the EOL string.    
    if (Serial.available() > 0) buffer.concat(char(Serial.read()));

    if (buffer.endsWith(EOL))
    {
      if (buffer.startsWith(String("ER"))){return MODULE_ERROR;}
      if (buffer.startsWith(String("OK"))){return SUCCESS;}
      buffer = "";
    }    
  } 
  return TIMEOUT_ERROR;  
}

// There are several commands that look for either OK or ERROR; let's abstract
//  support for those commands to one single private function, to save memory.
short stdCmd(String command)
{
  String buffer;
  
  knownStart(); // Clear the serial buffer in the module and the Arduino.
  
  Serial.print(command);
  Serial.print("\r");
  Serial.flush();
  
  // We're going to use the internal timer to track the elapsed time since we
  //  issued the command. Bog-standard Arduino stuff.
  unsigned long startTime = millis();
    
  // This is our timeout loop. We'll give the module 3 seconds.
  while ((startTime + 3000) > millis())
  {
    // Grow the current buffered data, until we receive the EOL string.    
    if (Serial.available() > 0) buffer.concat(char(Serial.read()));

    if (buffer.endsWith(EOL))
    {
      if (buffer.startsWith(String("ER"))) return MODULE_ERROR;
      if (buffer.startsWith(String("OK"))) return SUCCESS;
      buffer = "";
    }    
  }
  return TIMEOUT_ERROR;
}

// Similar to the command function, let's do a set parameter genrealization.
short stdSetParam(String command, String param)
{
  String buffer;
  
  knownStart();  // Clear Arduino and module serial buffers.
  
  Serial.print("SET ");
  Serial.print(command);
  Serial.print("=");
  Serial.print(param);
  Serial.print("\r");
  Serial.flush();
  
  // We're going to use the internal timer to track the elapsed time since we
  //  issued the reset. Bog-standard Arduino stuff.
  unsigned long startTime = millis();
  
  // This is our timeout loop. We'll give the module 2 seconds to reset.
  while ((startTime + 2000) > millis())
  {
    // Grow the current buffered data, until we receive the EOL string.    
    if (Serial.available() >0) buffer.concat(char(Serial.read()));

    if (buffer.endsWith(EOL))
    {
      if (buffer.startsWith(String("ER"))) return MODULE_ERROR;
      else if (buffer.startsWith(String("OK"))) return SUCCESS;
      buffer = "";
    }    
  }
  return TIMEOUT_ERROR;
}

void parser()
{
  String buffer;
  delay(10);
  while (Serial.available() > 0) 
  {
     char temp = char(Serial.read());
     buffer.concat(temp);
     if(buffer.endsWith(EOL))
     {
        break;
     } 
  }
  if(buffer.length()>0)
  {
    if(buffer.startsWith(String("OPEN_OK BLE")))
    {
      parserStatus = 1;
      strip.setPixelColor(status_led, 0x0000FF);
      strip.show();
    }
    if(buffer.startsWith(String("CLOSE_OK BLE")))
    {
      //parserStatus = 0;
      strip.setPixelColor(status_led, 0x000000);
      strip.show();
    }
    if(buffer.startsWith(String("RECV BLE +")))
    {
      switch(buffer[10])
      {
        case 'K':
          send_cromatica_CMD(BT_CNK);
          parserStatus = 1;
        break;
        case BT_RGB:
          if(mode==0)
          {
            RGBDH_w[3]=W_old;//=RGBWDH[3];
          }
          RGBWDH[0]=buffer[11];RGBWDH[1]=buffer[12];RGBWDH[2]=buffer[13]; //9+2 9+3 9+4
          RGBWDH[3]=0;
          mode=1;
          DIMMER = 238;
          staticShow();
          if(RGBWDH[0]!=0||RGBWDH[1]!=0||RGBWDH[2]!=0)
          RGBDH_r[0]=RGBWDH[0];RGBDH_r[1]=RGBWDH[1];RGBDH_r[2]=RGBWDH[2];RGBDH_r[3]=RGBWDH[3];
          effectStatus = 1;
        break;
        case BT_RGW:
          if(mode==1)
          {
            RGBWDH[3]=W_old;
          }          
          mode=0;
          RGBWDH[0]=buffer[11];RGBWDH[1]=buffer[12];RGBWDH[2]=buffer[13]; //9+2 9+3 9+4
          DIMMER = 238;
          staticShow();
          if(RGBWDH[0]!=0||RGBWDH[1]!=0||RGBWDH[2]!=0)
          RGBDH_w[0]=RGBWDH[0];RGBDH_w[1]=RGBWDH[1];RGBDH_w[2]=RGBWDH[2];
          //W_old=RGBWDH[3];
          effectStatus = 1;
        break;
        case BT_WHT:
          mode=0;
          RGBWDH[3]=buffer[11];
          if(RGBWDH[3]>1)W_old=RGBDH_w[3]=RGBWDH[3];
          DIMMER = 238;
          staticShow();
          effectStatus = 1;
        break;
        case BT_EFT:
          RGBWDH[0]=buffer[12]; // RED 9+3
          RGBWDH[1]=buffer[13]; // GREEN +4
          RGBWDH[2]=buffer[14]; // BLU +5
          RGBWDH[3]=buffer[11]; // WHITE +2
          RGBWDH[4]=buffer[16]; // DELAY +7
          RGBWDH[5]=buffer[17]; // HRZ +8
          RGBWDH[6]=buffer[15]; // VARIATION +6
          effectStatus = 2;
          effectON=1;
        break;
        case BT_PST:
          n_PE = 0;
          colorWipe(strip.Color(0,0,0));
          setPixel(buffer[11], buffer[12], myPreset[0].pixelsRGB);
          myPreset[0].r = buffer[13];
          myPreset[0].g = buffer[14];
          myPreset[0].b = buffer[15];
          myPreset[0].w = buffer[16];
          myPreset[0].d = buffer[17];
          N_PresetEffect = 1;
          effectON=1;
          effectStatus = 1;          
        break;
        case BT_SST:
          setPixel(buffer[11], buffer[12], myPreset[N_PresetEffect].pixelsRGB);
          myPreset[N_PresetEffect].r = buffer[13];
          myPreset[N_PresetEffect].g = buffer[14];
          myPreset[N_PresetEffect].b = buffer[15];
          myPreset[N_PresetEffect].w = buffer[16];
          myPreset[N_PresetEffect].d = buffer[17];
          effectON=1;
          N_PresetEffect++;
        break;
        case BT_FST:
          setPixel(buffer[11], buffer[12], myPreset[N_PresetEffect].pixelsRGB);
          myPreset[N_PresetEffect].r = buffer[13];
          myPreset[N_PresetEffect].g = buffer[14];
          myPreset[N_PresetEffect].b = buffer[15];
          myPreset[N_PresetEffect].w = buffer[16];
          myPreset[N_PresetEffect].d = buffer[17];
          N_PresetEffect++;
          effectON=1;
          effectStatus = 3;
        break;
        case BT_DNK:
          send_cromatica_CMD(BT_DNK);
//          parserStatus = 0;
        break;
        case BT_QRY:
        //...
        break;
      }
    }else
    {
//     Serial.print("Sono diverse");
    }
    if(buffer.startsWith(String("OPEN_OK A2DP")))
    {
      stdCmd("VOLUME 2 14");
      stdCmd("VOLUME 3 14");
    }
    if(buffer.startsWith(String("SCO_OPEN")))
    {
      stdCmd("PIO 3 1");
    }  
    if(buffer.startsWith(String("CALL_ACTIVE")))
    {
      stdCmd("PIO 3 1");
    }
    if(buffer.startsWith(String("CLOSE_OK A2DP")))
    {
      stdCmd("DISCOVERABLE ON");
    }
    if(buffer.startsWith(String("OPEN_OK TWS")))
    {
      stdCmd("DISCOVERABLE ON");
    }
  }
}

short BLEAdvertise()
{
  return stdCmd("ADVERTISING ON");
}

short BTdiscover()
{
  return stdCmd("DISCOVERABLE ON");
}


short twsforce()
{
  String buffer;

  knownStart(); // Clear the serial buffer in the module and the Arduino.
  Serial.print("TONE TE 400 V 64 TI 0 N C5 L 4 N R0 L 32 N C6 L 4\r");
//  delay(100);
//  Serial.print("UNPAIR 20FABB014F41\r"); //20FABB014F3D  20FABB014F41
  delay(500);
//  knownStart(); // Clear the serial buffer in the module and the Arduino.
  Serial.flush();

  Serial.print("OPEN 20FABB014FCD TWS\r"); //20FABB014F1B 20FABB014F40 20FABB014F3D 20FABB014FCD
  Serial.flush();
  delay(1000);
  unsigned long startTime = millis();
    
  // This is our timeout loop. We'll give the module 3 seconds.
  while ((startTime + 5000) > millis())
  {
    // Grow the current buffered data, until we receive the EOL string.    
    if (Serial.available() > 0) buffer.concat(char(Serial.read()));
    if (buffer.endsWith(EOL))
    {
      if (buffer.startsWith(String("ER")) || buffer.startsWith(String("OPEN_ERROR"))) return MODULE_ERROR;
      if (buffer.startsWith(String("PAIR"))) return SUCCESS;     
      buffer = "";
    }    
  }
  return TIMEOUT_ERROR;
}

short twspair()
{
  String buffer;

  knownStart(); // Clear the serial buffer in the module and the Arduino.
  Serial.print("TONE TE 400 V 64 TI 0 N C5 L 4 N R0 L 32 N C6 L 4\r");
  
//  knownStart(); // Clear the serial buffer in the module and the Arduino.
  Serial.flush();
  Serial.print("INQUIRY 38 TWS\r");
//  Serial.flush();
  delay(4000);
  
  
  // We're going to use the internal timer to track the elapsed time since we
  //  issued the command. Bog-standard Arduino stuff.
  unsigned long startTime = millis();
    
  // This is our timeout loop. We'll give the module 3 seconds.
  while ((startTime + 6000) > millis())
  {
    // Grow the current buffered data, until we receive the EOL string.    
    if (Serial.available() > 0) buffer.concat(char(Serial.read()));

    if (buffer.endsWith(EOL))
    {
      if(buffer.startsWith(String("INQUIRY ")))
      {
        MAC_BT_TWS=buffer.substring(8, 20);
        //knownStart(); 
        return SUCCESS;
      }
      if (buffer.startsWith(String("ER")))
      {
        buffer="";
      } 
      if (buffer.startsWith(String("OK")))
      {
        buffer="";
      }
    }    
  }
  return TIMEOUT_ERROR;
}
                     
short twsopen()
{
  String buffer;

//  knownStart(); // Clear the serial buffer in the module and the Arduino.
  
  Serial.print("OPEN ");
  Serial.print(MAC_BT_TWS);
  Serial.print(" TWS\r");
  Serial.flush();
  delay(1000);
  unsigned long startTime = millis();
    
  // This is our timeout loop. We'll give the module 3 seconds.
  while ((startTime + 5000) > millis())
  {
    // Grow the current buffered data, until we receive the EOL string.    
    if (Serial.available() > 0) buffer.concat(char(Serial.read()));
    if (buffer.endsWith(EOL))
    {
      if (buffer.startsWith(String("ER")) || buffer.startsWith(String("OPEN_ERROR"))) return MODULE_ERROR;
      if (buffer.startsWith(String("PAIR"))) return SUCCESS;     
      buffer = "";
    }    
  }
  return TIMEOUT_ERROR;
}
